@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/Output/Template/Document.pm - Template Toolkit document
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::Output::Template::Document;
@@ -12,9 +13,7 @@ package Kernel::Output::Template::Document;
 use strict;
 use warnings;
 
-use parent qw (Template::Document);
-
-our $ObjectManagerDisabled = 1;
+use base qw (Template::Document);
 
 =head1 NAME
 
@@ -22,7 +21,11 @@ Kernel::Output::Template::Document - Template Toolkit document extension package
 
 =head1 PUBLIC INTERFACE
 
-=head2 process()
+=over 4
+
+=cut
+
+=item process()
 
 process this template document. This method is inherited from
 Template::Document and used to perform some up-front initialization
@@ -35,14 +38,11 @@ sub process {
 
     $Self->_InstallOTRSExtensions($Context);
     $Self->_PrecalculateBlockStructure($Context);
-    $Self->_PrecalculateBlockHookSubscriptions($Context);
 
     return $Self->SUPER::process($Context);
 }
 
-=begin Internal:
-
-=head2 _InstallOTRSExtensions()
+=item _InstallOTRSExtensions()
 
 adds some OTRS specific extensions to Template::Toolkit.
 
@@ -72,26 +72,20 @@ sub _InstallOTRSExtensions {
             my $output = '';
             my ( %_tt_args, $_tt_params );
             $_tt_args{'BlockName'} = shift;
-            $_tt_params            = shift;
-            $_tt_params            = {} if ref $_tt_params ne 'HASH';
-            $_tt_params            = { %_tt_args, %$_tt_params };
+            $_tt_params = shift;
+            $_tt_params = {} if ref $_tt_params ne 'HASH';
+            $_tt_params = { %_tt_args, %$_tt_params };
 
             my $stash = $Context->localise($_tt_params);
             eval {
 
-                my $BlockName   = $stash->get('BlockName');
+                my $BlockName = $stash->get('BlockName');
                 my $ParentBlock = $stash->get('ParentBlock') || $stash->{_BlockTree};
 
                 return if !exists $ParentBlock->{Children};
                 return if !exists $ParentBlock->{Children}->{$BlockName};
 
-                my $TemplateName = $stash->get('template')->{name} // '';
-                $TemplateName = substr( $TemplateName, 0, -3 );    # remove .tt extension
-                my $GenerateBlockHook =
-                    $Context->{LayoutObject}->{_BlockHookSubscriptions}->{$TemplateName}->{$BlockName};
-
                 for my $TargetBlock ( @{ $ParentBlock->{Children}->{$BlockName} } ) {
-                    $output .= "<!--HookStart${BlockName}-->\n" if $GenerateBlockHook;
                     $output .= $Context->process(
                         $TargetBlock->{Path},
                         {
@@ -99,7 +93,6 @@ sub _InstallOTRSExtensions {
                             'ParentBlock' => $TargetBlock,
                         },
                     );
-                    $output .= "<!--HookEnd${BlockName}-->\n" if $GenerateBlockHook;
                 }
                 delete $ParentBlock->{Children}->{$BlockName};
 
@@ -108,7 +101,7 @@ sub _InstallOTRSExtensions {
 
             die $@ if $@;
             return $output;
-        }
+            }
     );
 
     #
@@ -116,9 +109,10 @@ sub _InstallOTRSExtensions {
     #     footer of the page, all in one place.
     #
     # Usage:
-    #     [% WRAPPER JSOnDocumentComplete -%]
+    #     [% WRAPPER JSOnDocumentComplete -% ]
     #         console.log();
-    #     [%- END %]
+    #     [%- END % ]
+    # %]
     #
 
     $Self->{_DEFBLOCKS}->{JSOnDocumentComplete} //= sub {
@@ -179,83 +173,10 @@ sub _InstallOTRSExtensions {
         return $output;
     };
 
-    #
-    # This block is used to cut out JavaScript data that needs to be inserted to Core.Config
-    #   from the templates and insert it in the footer of the page, all in one place.
-    #
-    # Usage:
-    #     [% Process JSData
-    #         Key   = 'Test.Key'
-    #         Value = { ... }
-    #     %]
-    #
-    #
-
-    $Self->{_DEFBLOCKS}->{JSData} //= sub {
-        my $context = shift || die "template sub called without context\n";
-        my $stash   = $context->stash();
-        my $output  = '';
-
-        my $_tt_error;
-
-        eval {
-
-            my $Key   = $stash->get('Key');
-            my $Value = $stash->get('Value');
-
-            return $output if !$Key;
-
-            $context->{LayoutObject}->{_JSData} //= {};
-            $context->{LayoutObject}->{_JSData}->{$Key} = $Value;
-
-        };
-        if ($@) {
-            $_tt_error = $context->catch( $@, \$output );
-            die $_tt_error if $_tt_error->type() ne 'return';
-        }
-
-        return $output;
-    };
-
-    #
-    # This block is used to insert the collected JavaScript data in the page footer.
-    #
-
-    $Self->{_DEFBLOCKS}->{JSDataInsert} //= sub {
-        my $context = shift || die "template sub called without context\n";
-        my $stash   = $context->stash();
-        my $output  = '';
-        my $_tt_error;
-
-        eval {
-            my %Data = %{ $context->{LayoutObject}->{_JSData} // {} };
-            if (%Data) {
-                my $JSONString = $Kernel::OM->Get('Kernel::System::JSON')->Encode(
-                    Data     => \%Data,
-                    SortKeys => 1,
-                );
-
-                # Escape closing script tags in the JSON content as they will confuse the
-                #   browser's parser.
-                $JSONString =~ s{ </(?<ScriptTag>script)}{<\\/$+{ScriptTag}}ismxg;
-
-                $output .= "Core.Config.AddConfig($JSONString);\n";
-            }
-            delete $context->{LayoutObject}->{_JSData};
-        };
-
-        if ($@) {
-            $_tt_error = $context->catch( $@, \$output );
-            die $_tt_error if $_tt_error->type() ne 'return';
-        }
-
-        return $output;
-    };
-
     return;
 }
 
-=head2 _PrecalculateBlockStructure()
+=item _PrecalculateBlockStructure()
 
 pre-calculates the tree structure of the blocks so that it
 can be used by PerformRenderBlock in an efficient way.
@@ -265,6 +186,9 @@ can be used by PerformRenderBlock in an efficient way.
 sub _PrecalculateBlockStructure {
     my ( $Self, $Context ) = @_;
 
+    #
+    # TODO cache result in current object?
+    #
     my $Defblocks = $Self->{_DEFBLOCKS} || {};
 
     my $BlockData = $Context->stash()->get( [ 'global', 0, 'BlockData', 0 ] ) || [];
@@ -276,10 +200,10 @@ sub _PrecalculateBlockStructure {
 
     BLOCKPATHIDENTIFIER:
     for my $BlockIdentifier ( sort keys %{$Defblocks} ) {
-        my @BlockPath       = split( m{/}, $BlockIdentifier );
+        my @BlockPath = split( m{/}, $BlockIdentifier );
         my $BlockPathLength = scalar @BlockPath;
         next BLOCKPATHIDENTIFIER if !$BlockPathLength;
-        $BlockPaths->{ $BlockPath[-1] }   = $BlockIdentifier;
+        $BlockPaths->{ $BlockPath[-1] } = $BlockIdentifier;
         $BlockParents->{ $BlockPath[-1] } = [ splice( @BlockPath, 0, $BlockPathLength - 1 ) ];
     }
 
@@ -330,43 +254,16 @@ sub _PrecalculateBlockStructure {
     return;
 }
 
-=head2 _PrecalculateBlockHookSubscriptions()
-
-=cut
-
-sub _PrecalculateBlockHookSubscriptions {
-    my ( $Self, $Context ) = @_;
-
-    # Only calculate once per LayoutObject
-    return if defined $Context->{LayoutObject}->{_BlockHookSubscriptions};
-
-    my $Config = $Kernel::OM->Get('Kernel::Config')->Get('Frontend::Template::GenerateBlockHooks') // {};
-
-    my %BlockHooks;
-
-    for my $Key ( sort keys %{ $Config // {} } ) {
-        for my $Template ( sort keys %{ $Config->{$Key} // {} } ) {
-            for my $Block ( @{ $Config->{$Key}->{$Template} // [] } ) {
-                $BlockHooks{$Template}->{$Block} = 1;
-            }
-        }
-    }
-
-    $Context->{LayoutObject}->{_BlockHookSubscriptions} = \%BlockHooks;
-
-    return;
-}
-
 1;
 
-=end Internal:
+=back
 
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
