@@ -1,9 +1,10 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Kernel/System/Ticket/TicketACL.pm - all ticket ACL functions
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::Ticket::TicketACL;
@@ -19,12 +20,15 @@ our $ObjectManagerDisabled = 1;
 
 Kernel::System::Ticket::TicketACL - ticket ACL lib
 
-=head1 DESCRIPTION
+=head1 SYNOPSIS
 
 All ticket ACL functions.
 
+=over 4
 
-=head2 TicketAcl()
+=cut
+
+=item TicketAcl()
 
 Restricts the Data parameter sent to a subset of it, depending on a group of user defied rules
 called ACLs. The reduced subset can be access from TicketACLData() if ReturnType parameter is set
@@ -55,8 +59,6 @@ Example to restrict ticket actions:
 
         QueueID          => 123,                      # Optional
         Queue            => 'some queue name',        # Optional
-        NewQueueID       => 123,                      # Optional, QueueID or NewQueueID can be
-                                                      #   used and they both refers to QueueID
 
         ServiceID        => 123,                      # Optional
         Service          => 'some service name',      # Optional
@@ -424,9 +426,6 @@ sub TicketAcl {
                     for my $Item ( @{ $Step{$PropertiesHash}->{$Key}->{$Data} } ) {
                         if ( ref $UsedChecks{$Key}->{$Data} eq 'ARRAY' ) {
                             my $MatchItem = 0;
-                            if ( substr( $Item, 0, length '[Not' ) eq '[Not' ) {
-                                $MatchItem = 1;
-                            }
                             my $MatchedArrayDataItem;
                             ARRAYDATAITEM:
                             for my $ArrayDataItem ( @{ $UsedChecks{$Key}->{$Data} } ) {
@@ -919,7 +918,7 @@ sub TicketAcl {
     return 1;
 }
 
-=head2 TicketAclData()
+=item TicketAclData()
 
 return the current ACL data hash after TicketAcl()
 
@@ -933,7 +932,7 @@ sub TicketAclData {
     return %{ $Self->{TicketAclData} || {} };
 }
 
-=head2 TicketAclActionData()
+=item TicketAclActionData()
 
 return the current ACL action data hash after TicketAcl()
 
@@ -954,7 +953,7 @@ sub TicketAclActionData {
 
 =cut
 
-=head2 _GetChecks()
+=item _GetChecks()
 
 creates two check hashes (one for current data updatable via AJAX refreshes and another for
 static ticket data stored in the DB) with the required data to use as a basis to match the ACLs
@@ -1132,19 +1131,6 @@ sub _GetChecks {
         }
     }
 
-    # Check for ActivityEntityID if set as parameter (ProcessManagement).
-    if ( ( $CheckAll || $RequiredChecks{Process} ) && $Param{ActivityEntityID} ) {
-
-        my $Activity = $Kernel::OM->Get('Kernel::System::ProcessManagement::Activity')->ActivityGet(
-            ActivityEntityID => $Param{ActivityEntityID},
-            Interface        => [$Interface],
-        );
-
-        if ( IsHashRefWithData($Activity) ) {
-            $Checks{Process}->{ActivityEntityID} = $Param{ActivityEntityID};
-        }
-    }
-
     # check for dynamic fields
     if ( IsHashRefWithData( $Param{DynamicField} ) ) {
         $Checks{DynamicField} = $Param{DynamicField};
@@ -1164,27 +1150,17 @@ sub _GetChecks {
 
         # check if is a dynamic field with data
         next TICKETATTRIBUTE if $TicketAttribute !~ m{ \A DynamicField_ }smx;
-        next TICKETATTRIBUTE if !defined $Checks{Ticket}->{$TicketAttribute};
-        next TICKETATTRIBUTE if !length $Checks{Ticket}->{$TicketAttribute};
-
-        if (
+        next TICKETATTRIBUTE if !$Checks{Ticket}->{$TicketAttribute};
+        next TICKETATTRIBUTE if
             ref $Checks{Ticket}->{$TicketAttribute} eq 'ARRAY'
-            && !IsArrayRefWithData( $Checks{Ticket}->{$TicketAttribute} )
-            )
-        {
-            next TICKETATTRIBUTE;
-        }
+            && !IsArrayRefWithData( $Checks{Ticket}->{$TicketAttribute} );
 
         # compare if data is different and skip on same data
-        if (
-            $Checks{DynamicField}->{$TicketAttribute}
-            && !DataIsDifferent(
+        if ( $Checks{DynamicField}->{$TicketAttribute} ) {
+            next TICKETATTRIBUTE if !DataIsDifferent(
                 Data1 => $Checks{Ticket}->{$TicketAttribute},
                 Data2 => $Checks{DynamicField}->{$TicketAttribute},
-            )
-            )
-        {
-            next TICKETATTRIBUTE;
+            );
         }
 
         $Checks{DynamicField}->{$TicketAttribute} = $Checks{Ticket}->{$TicketAttribute};
@@ -1197,16 +1173,10 @@ sub _GetChecks {
 
         # check if is a dynamic field with data
         next TICKETATTRIBUTE if $TicketAttribute !~ m{ \A DynamicField_ }smx;
-        next TICKETATTRIBUTE if !defined $ChecksDatabase{Ticket}->{$TicketAttribute};
-        next TICKETATTRIBUTE if !length $ChecksDatabase{Ticket}->{$TicketAttribute};
-
-        if (
+        next TICKETATTRIBUTE if !$ChecksDatabase{Ticket}->{$TicketAttribute};
+        next TICKETATTRIBUTE if
             ref $ChecksDatabase{Ticket}->{$TicketAttribute} eq 'ARRAY'
-            && !IsArrayRefWithData( $ChecksDatabase{Ticket}->{$TicketAttribute} )
-            )
-        {
-            next TICKETATTRIBUTE;
-        }
+            && !IsArrayRefWithData( $ChecksDatabase{Ticket}->{$TicketAttribute} );
 
         $ChecksDatabase{DynamicField}->{$TicketAttribute} = $ChecksDatabase{Ticket}->{$TicketAttribute};
     }
@@ -1222,24 +1192,29 @@ sub _GetChecks {
         my $GroupObject = $Kernel::OM->Get('Kernel::System::Group');
 
         for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-            my %Groups = $GroupObject->PermissionUserGet(
+            my @Groups = $GroupObject->GroupMemberList(
                 UserID => $Param{UserID},
+                Result => 'Name',
                 Type   => $Type,
             );
-
-            my @GroupNames = sort values %Groups;
-
-            $User{"Group_$Type"} = \@GroupNames;
+            $User{"Group_$Type"} = \@Groups;
         }
 
-        my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+        my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
             UserID => $Param{UserID},
+            Result => 'ID',
         );
+        my @Roles;
+        ROLEID:
+        for my $RoleID (@RoleIDs) {
+            my $RoleName = $GroupObject->RoleLookup(
+                RoleID => $RoleID,
+            );
+            next ROLEID if !$RoleName;
+            push @Roles, $RoleName;
+        }
+        $User{Role} = \@Roles;
 
-        my @RoleNames = sort values %RoleIDs;
-
-        $User{Role}           = \@RoleNames;
         $Checks{User}         = \%User;
         $ChecksDatabase{User} = \%User;
     }
@@ -1262,7 +1237,6 @@ sub _GetChecks {
                 Result => 'Name',
                 Type   => $Type,
             );
-
             $CustomerUser{"Group_$Type"} = \@Groups;
         }
 
@@ -1339,10 +1313,6 @@ sub _GetChecks {
         # get queue object
         my $QueueObject = $Kernel::OM->Get('Kernel::System::Queue');
 
-        if ( $Param{NewQueueID} && !$Param{QueueID} ) {
-            $Param{QueueID} = $Param{NewQueueID};
-        }
-
         if ( $Param{QueueID} ) {
             my %Queue = $QueueObject->QueueGet( ID => $Param{QueueID} );
             $Checks{Queue} = \%Queue;
@@ -1550,7 +1520,7 @@ sub _GetChecks {
 
         # use priority data (if given)
         if ( $Param{NewPriorityID} && !$Param{PriorityID} ) {
-            $Param{PriorityID} = $Param{NewPriorityID};
+            $Param{PriorityID} = $Param{NewPriorityID}
         }
 
         if ( $Param{PriorityID} ) {
@@ -1691,8 +1661,8 @@ sub _GetChecks {
 
     # use state data (if given)
     if ( $CheckAll || $RequiredChecks{State} ) {
-        if ( !$Param{StateID} ) {
-            $Param{StateID} = $Param{NextStateID} || $Param{NewStateID};
+        if ( $Param{NextStateID} && !$Param{StateID} ) {
+            $Param{StateID} = $Param{NextStateID}
         }
         if ( $Param{StateID} ) {
             my %State = $StateObject->StateGet(
@@ -1784,24 +1754,29 @@ sub _GetChecks {
                 UserID => $Param{OwnerID},
             );
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $Param{OwnerID},
+                    Result => 'Name',
                     Type   => $Type,
                 );
-
-                my @GroupNames = sort values %Groups;
-
-                $Owner{"Group_$Type"} = \@GroupNames;
+                $Owner{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $Param{OwnerID},
+                Result => 'ID',
             );
+            my @Roles;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+                next ROLEID if !$RoleName;
+                push @Roles, $RoleName;
+            }
+            $Owner{Role} = \@Roles;
 
-            my @RoleNames = sort values %RoleIDs;
-
-            $Owner{Role}   = \@RoleNames;
             $Checks{Owner} = \%Owner;
 
             # update or add owner information to the ticket check
@@ -1816,24 +1791,29 @@ sub _GetChecks {
                 UserID => $OwnerID,
             );
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $OwnerID,
+                    Result => 'Name',
                     Type   => $Type,
                 );
-
-                my @GroupNames = sort values %Groups;
-
-                $Owner{"Group_$Type"} = \@GroupNames;
+                $Owner{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $OwnerID,
+                Result => 'ID',
             );
+            my @Roles;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+                next ROLEID if !$RoleName;
+                push @Roles, $RoleName;
+            }
+            $Owner{Role} = \@Roles;
 
-            my @RoleNames = sort values %RoleIDs;
-
-            $Owner{Role}   = \@RoleNames;
             $Checks{Owner} = \%Owner;
 
             # update or add owner information to the ticket check
@@ -1848,24 +1828,29 @@ sub _GetChecks {
                     UserID => $Checks{Ticket}->{OwnerID},
                 );
                 for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                    my %Groups = $GroupObject->PermissionUserGet(
+                    my @Groups = $GroupObject->GroupMemberList(
                         UserID => $Checks{Ticket}->{OwnerID},
+                        Result => 'Name',
                         Type   => $Type,
                     );
-
-                    my @GroupNames = sort values %Groups;
-
-                    $Owner{"Group_$Type"} = \@GroupNames;
+                    $Owner{"Group_$Type"} = \@Groups;
                 }
 
-                my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+                my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                     UserID => $Checks{Ticket}->{OwnerID},
+                    Result => 'ID',
                 );
+                my @Roles;
+                ROLEID:
+                for my $RoleID (@RoleIDs) {
+                    my $RoleName = $GroupObject->RoleLookup(
+                        RoleID => $RoleID,
+                    );
+                    next ROLEID if !$RoleName;
+                    push @Roles, $RoleName;
+                }
+                $Owner{Role} = \@Roles;
 
-                my @RoleNames = sort values %RoleIDs;
-
-                $Owner{Role}   = \@RoleNames;
                 $Checks{Owner} = \%Owner;
             }
         }
@@ -1889,24 +1874,29 @@ sub _GetChecks {
                 UserID => $ChecksDatabase{Ticket}->{OwnerID},
             );
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $ChecksDatabase{Ticket}->{OwnerID},
+                    Result => 'Name',
                     Type   => $Type,
                 );
-
-                my @GroupNames = sort values %Groups;
-
-                $Owner{"Group_$Type"} = \@GroupNames;
+                $Owner{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $ChecksDatabase{Ticket}->{OwnerID},
+                Result => 'ID',
             );
+            my @Roles;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+                next ROLEID if !$RoleName;
+                push @Roles, $RoleName;
+            }
+            $Owner{Role} = \@Roles;
 
-            my @RoleNames = sort values %RoleIDs;
-
-            $Owner{Role}           = \@RoleNames;
             $ChecksDatabase{Owner} = \%Owner;
         }
     }
@@ -1920,24 +1910,29 @@ sub _GetChecks {
                 UserID => $Param{ResponsibleID},
             );
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $Param{ResponsibleID},
+                    Result => 'Name',
                     Type   => $Type,
                 );
-
-                my @GroupNames = sort values %Groups;
-
-                $Responsible{"Group_$Type"} = \@GroupNames;
+                $Responsible{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $Param{ResponsibleID},
+                Result => 'ID',
             );
+            my @Roles;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+                next ROLEID if !$RoleName;
+                push @Roles, $RoleName;
+            }
+            $Responsible{Role} = \@Roles;
 
-            my @RoleNames = sort values %RoleIDs;
-
-            $Responsible{Role}   = \@RoleNames;
             $Checks{Responsible} = \%Responsible;
 
             # update or add responsible information to the ticket check
@@ -1952,24 +1947,29 @@ sub _GetChecks {
                 UserID => $ResponsibleID,
             );
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $ResponsibleID,
+                    Result => 'Name',
                     Type   => $Type,
                 );
-
-                my @GroupNames = sort values %Groups;
-
-                $Responsible{"Group_$Type"} = \@GroupNames;
+                $Responsible{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $ResponsibleID,
+                Result => 'ID',
             );
+            my @Roles;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+                next ROLEID if !$RoleName;
+                push @Roles, $RoleName;
+            }
+            $Responsible{Role} = \@Roles;
 
-            my @RoleNames = sort values %RoleIDs;
-
-            $Responsible{Role}   = \@RoleNames;
             $Checks{Responsible} = \%Responsible;
 
             # update or add responsible information to the ticket check
@@ -1984,24 +1984,29 @@ sub _GetChecks {
                     UserID => $Checks{Ticket}->{ResponsibleID},
                 );
                 for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
-
-                    my %Groups = $GroupObject->PermissionUserGet(
+                    my @Groups = $GroupObject->GroupMemberList(
                         UserID => $Checks{Ticket}->{ResponsibleID},
+                        Result => 'Name',
                         Type   => $Type,
                     );
-
-                    my @GroupNames = sort values %Groups;
-
-                    $Responsible{"Group_$Type"} = \@GroupNames;
+                    $Responsible{"Group_$Type"} = \@Groups;
                 }
 
-                my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+                my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                     UserID => $Checks{Ticket}->{ResponsibleID},
+                    Result => 'ID',
                 );
+                my @Roles;
+                ROLEID:
+                for my $RoleID (@RoleIDs) {
+                    my $RoleName = $GroupObject->RoleLookup(
+                        RoleID => $RoleID,
+                    );
+                    next ROLEID if !$RoleName;
+                    push @Roles, $RoleName;
+                }
+                $Responsible{Role} = \@Roles;
 
-                my @RoleNames = sort values %RoleIDs;
-
-                $Responsible{Role}   = \@RoleNames;
                 $Checks{Responsible} = \%Responsible;
             }
         }
@@ -2029,23 +2034,35 @@ sub _GetChecks {
 
             for my $Type ( @{ $ConfigObject->Get('System::Permission') } ) {
 
-                my %Groups = $GroupObject->PermissionUserGet(
+                my @Groups = $GroupObject->GroupMemberList(
                     UserID => $ChecksDatabase{Ticket}->{ResponsibleID},
+                    Result => 'Name',
                     Type   => $Type,
                 );
 
-                my @GroupNames = sort values %Groups;
-
-                $Responsible{"Group_$Type"} = \@GroupNames;
+                $Responsible{"Group_$Type"} = \@Groups;
             }
 
-            my %RoleIDs = $GroupObject->PermissionUserRoleGet(
+            my @RoleIDs = $GroupObject->GroupUserRoleMemberList(
                 UserID => $ChecksDatabase{Ticket}->{ResponsibleID},
+                Result => 'ID',
             );
+            my @Roles;
 
-            my @RoleNames = sort values %RoleIDs;
+            ROLEID:
+            for my $RoleID (@RoleIDs) {
+
+                my $RoleName = $GroupObject->RoleLookup(
+                    RoleID => $RoleID,
+                );
+
+                next ROLEID if !$RoleName;
+
+                push @Roles, $RoleName;
+            }
+
+            $Responsible{Role} = \@Roles;
 
-            $Responsible{Role}           = \@RoleNames;
             $ChecksDatabase{Responsible} = \%Responsible;
         }
     }
@@ -2061,7 +2078,7 @@ sub _GetChecks {
     };
 }
 
-=head2 _CompareMatchWithData()
+=item _CompareMatchWithData()
 
 Compares a properties element with the data sent to the ACL, the compare results varies on how the
 ACL properties where defined including normal, negated, regular expression and negated regular
@@ -2103,42 +2120,15 @@ sub _CompareMatchWithData {
     my $Match = $Param{Match};
     my $Data  = $Param{Data};
 
-    # Negated matches requires a different logic.
-    if ( substr( $Match, 0, length '[Not' ) eq '[Not' ) {
-
-        # Not equal match
-        if ( substr( $Match, 0, length '[Not]' ) eq '[Not]' ) {
-            my $NotValue = substr $Match, length '[Not]';
-            if ( $NotValue eq $Data ) {
-                return {
-                    Success => 1,
-                    Match   => 0,
-                };
-            }
+    # Not equal match, this case requires a reverse logic than the rest
+    if ( substr( $Match, 0, length '[Not]' ) eq '[Not]' ) {
+        my $NotValue = substr $Match, length '[Not]';
+        if ( $NotValue eq $Data ) {
+            return {
+                Success => 1,
+                Match   => 0,
+            };
         }
-
-        # Not reg-exp match case-sensitive.
-        elsif ( substr( $Match, 0, length '[NotRegExp]' ) eq '[NotRegExp]' ) {
-            my $RegExp = substr $Match, length '[NotRegExp]';
-            if ( $Data =~ /$RegExp/ ) {
-                return {
-                    Success => 1,
-                    Match   => 0,
-                };
-            }
-        }
-
-        # Not reg-exp match case-insensitive.
-        elsif ( substr( $Match, 0, length '[Notregexp]' ) eq '[Notregexp]' ) {
-            my $RegExp = substr $Match, length '[Notregexp]';
-            if ( $Data =~ /$RegExp/i ) {
-                return {
-                    Success => 1,
-                    Match   => 0,
-                };
-            }
-        }
-
         if ( $Param{SingleItem} ) {
             return {
                 Success => 1,
@@ -2154,65 +2144,87 @@ sub _CompareMatchWithData {
             };
         }
     }
-    else {
 
-        # Equal match.
-        if ( $Match eq $Data ) {
+    # equal match
+    elsif ( $Match eq $Data ) {
+        return {
+            Success => 1,
+            Match   => 1,
+        };
+    }
+
+    # reg-exp match case-sensitive
+    elsif ( substr( $Match, 0, length '[RegExp]' ) eq '[RegExp]' ) {
+        my $RegExp = substr $Match, length '[RegExp]';
+        if ( $Data =~ /$RegExp/ ) {
             return {
                 Success => 1,
                 Match   => 1,
             };
         }
+    }
 
-        # Reg-exp match case-sensitive.
-        elsif ( substr( $Match, 0, length '[RegExp]' ) eq '[RegExp]' ) {
-            my $RegExp = substr $Match, length '[RegExp]';
-            if ( $Data =~ /$RegExp/ ) {
-                return {
-                    Success => 1,
-                    Match   => 1,
-                };
-            }
-        }
-
-        # Reg-exp match case-insensitive.
-        elsif ( substr( $Match, 0, length '[regexp]' ) eq '[regexp]' ) {
-            my $RegExp = substr $Match, length '[regexp]';
-            if ( $Data =~ /$RegExp/i ) {
-                return {
-                    Success => 1,
-                    Match   => 1,
-                };
-            }
-        }
-
-        if ( $Param{SingleItem} ) {
+    # reg-exp match case-insensitive
+    elsif ( substr( $Match, 0, length '[regexp]' ) eq '[regexp]' ) {
+        my $RegExp = substr $Match, length '[regexp]';
+        if ( $Data =~ /$RegExp/i ) {
             return {
                 Success => 1,
-                Match   => 0,
-                Skip    => 0,
+                Match   => 1,
             };
         }
-        else {
+    }
+
+    # not reg-exp match case-sensitive
+    elsif ( substr( $Match, 0, length '[NotRegExp]' ) eq '[NotRegExp]' ) {
+        my $RegExp = substr $Match, length '[NotRegExp]';
+        if ( $Data !~ /$RegExp/ ) {
             return {
                 Success => 1,
-                Match   => 0,
-                Skip    => 1,
+                Match   => 1,
             };
         }
     }
+
+    # not reg-exp match case-insensitive
+    elsif ( substr( $Match, 0, length '[Notregexp]' ) eq '[Notregexp]' ) {
+        my $RegExp = substr $Match, length '[Notregexp]';
+        if ( $Data !~ /$RegExp/i ) {
+            return {
+                Success => 1,
+                Match   => 1,
+            };
+        }
+    }
+
+    if ( $Param{SingleItem} ) {
+        return {
+            Success => 1,
+            Match   => 0,
+            Skip    => 0,
+        };
+    }
+    else {
+        return {
+            Success => 1,
+            Match   => 0,
+            Skip    => 1,
+        };
+    }
 }
 
 1;
 
 =end Internal:
 
+=back
+
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
