@@ -1,9 +1,7 @@
-#!/usr/bin/perl -w
+#!/usr/bin/perl
 # --
 # bin/fcgi-bin/public.pl - the global FastCGI handle file for OTRS
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
-# --
-# $Id: public.pl,v 1.4 2011-03-08 14:15:54 mb Exp $
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU AFFERO General Public License as published by
@@ -30,14 +28,12 @@ use lib "$Bin/../..";
 use lib "$Bin/../../Kernel/cpan-lib";
 use lib "$Bin/../../Custom";
 
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.4 $) [1];
-
 # Imports the library; required line
 use CGI::Fast;
 
 # load agent web interface
 use Kernel::System::Web::InterfacePublic();
+use Kernel::System::ObjectManager;
 
 # 0=off;1=on;
 my $Debug = 0;
@@ -47,11 +43,11 @@ my $Debug = 0;
 # Response loop
 while ( my $WebRequest = new CGI::Fast ) {
 
-    # create new object
-    my $Interface
-        = Kernel::System::Web::InterfacePublic->new( Debug => $Debug, WebRequest => $WebRequest );
-
-    # execute object
+    local $Kernel::OM = Kernel::System::ObjectManager->new();
+    my $Interface = Kernel::System::Web::InterfacePublic->new(
+        Debug      => $Debug,
+        WebRequest => $WebRequest,
+    );
     $Interface->Run();
 
     #    $Cnt++;
